## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(warning = FALSE, message = FALSE, crop=NULL)

## ----loading packages, echo=TRUE----------------------------------------------
library(BCalm)
library(dplyr)
library(ggplot2)
library(kableExtra) # for visually appealing tables

## ----First dataset------------------------------------------------------------
data("BcSetExample")
nr_reps = 3
# show the data
kable(head(BcSetExample), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")

## ----Variant Map--------------------------------------------------------------
# load the variant map
data("MapExample")
# show the data
kable(head(MapExample), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")

# create the variant dataframe by adding the variant ID to the DNA and RNA counts
var_df <- create_var_df(BcSetExample, MapExample)
# show the data
kable(head(var_df), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")


## ----Variant Downsampling-----------------------------------------------------
var_df <- downsample_barcodes(var_df, id_column_name="variant_id")

## ----Creating dna dataframe from var_df---------------------------------------
dna_var <- create_dna_df(var_df)
kable(head(dna_var), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")

## ----Creating rna dataframe from var_df---------------------------------------
rna_var <- create_rna_df(var_df)
kable(head(rna_var), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")

## ----Variant MPRASet----------------------------------------------------------
# create the variant specific MPRAset
BcVariantMPRASetExample <- MPRASet(DNA = dna_var, RNA = rna_var, eid = row.names(dna_var), barcode = NULL)

## ----LabelsVec----------------------------------------------------------------
data(LabelExample)
table(LabelExample)
kable(head(LabelExample), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")

## ----Downsampling-------------------------------------------------------------
elem_df <- downsample_barcodes(BcSetExample)

## ----Creating dataframes------------------------------------------------------
dna_elem <- create_dna_df(elem_df, id_column_name="name")
rna_elem <- create_rna_df(elem_df, id_column_name="name")

## ----Element MPRASet----------------------------------------------------------
BcLabelMPRASetExample <- MPRASet(DNA = dna_elem, RNA = rna_elem, eid = row.names(dna_elem), barcode = NULL, label=LabelExample)

## ----Fit Variants-------------------------------------------------------------
bcs <- ncol(dna_var) / nr_reps
design <- data.frame(intcpt = 1, alt = grepl("alt", colnames(BcVariantMPRASetExample)))
block_vector <- rep(1:nr_reps, each=bcs)
mpralm_fit_var <- mpralm(object = BcVariantMPRASetExample, design = design, aggregate = "none", normalize = TRUE, model_type = "corr_groups", plot = FALSE, block = block_vector)

top_var <- topTable(mpralm_fit_var, coef = 2, number = Inf)
kable(head(rna_var), "html") %>% kable_styling("striped") %>% scroll_box(width = "100%")

## ----Volcano Plot-------------------------------------------------------------
ggplot(top_var, aes(x = logFC, y = -log10(P.Value))) +
  geom_point(alpha = 0.6)

## ----Fit elements, fig.width=8, fig.height=4----------------------------------
bcs <- ncol(dna_elem) / nr_reps
block_vector <- rep(1:nr_reps, each=bcs)
mpralm_fit_elem <- fit_elements(object = BcLabelMPRASetExample, normalize=TRUE, block = block_vector, plot = FALSE)

## ----Visualization, fig=TRUE, fig.width=8, fig.height=4, warning=FALSE--------
plot_groups(mpralm_fit_elem, 0.975, neg_label="control", test_label="tested")

## ----MPRA treat---------------------------------------------------------------
treat <- mpra_treat(mpralm_fit_elem, 0.975, neg_label="control")
result <- topTreat(treat, coef = 1, number = Inf)
head(result)

## ----sessionInfo, results='asis', echo=FALSE----------------------------------
sessionInfo()

